
#include <argos3/core/simulator/simulator.h>
#include <argos3/core/simulator/entity/composable_entity.h>
#include <argos3/core/simulator/entity/embodied_entity.h>
#include <argos3/core/simulator/entity/floor_entity.h>
#include <../../sensor_entity/phero_sensor_equipped_entity/phero_sensor_equipped_entity.h>

#include "footbot_phero_sensor.h"

namespace argos {

 /****************************************/
 /****************************************/

 static CRange<Real> UNIT(0.0f, 1.0f);

 /****************************************/
 /****************************************/

 CFootBotPheroSensor::CFootBotPheroSensor() :
 m_pcLoopFunctions(dynamic_cast<CPheroLoopFunctions&>(CSimulator::GetInstance().GetLoopFunctions())),
 m_pcEmbodiedEntity(NULL),
 //m_pcFloorEntity(NULL),
 m_pcPheroSensorEntity(NULL),
 //m_pcRNG(NULL),
 //m_bAddNoise(false),
 m_cSpace(CSimulator::GetInstance().GetSpace()) {}

 /****************************************/
 /****************************************/

 void CFootBotPheroSensor::SetRobot(CComposableEntity& c_entity) {
 m_pcEmbodiedEntity = &(c_entity.GetComponent<CEmbodiedEntity>("body"));
 m_pcPheroSensorEntity = &(c_entity.GetComponent<CPheroSensorEquippedEntity>("phero_sensors"));
 m_pcPheroSensorEntity->Enable();
 //m_pcFloorEntity = &m_cSpace.GetFloorEntity();
 }

 /****************************************/
 /****************************************/

 void CFootBotPheroSensor::Init(TConfigurationNode& t_tree) {
 try {
 CCI_FootBotPheroSensor::Init(t_tree);
 /* Parse noise level */
 //Real fNoiseLevel = 0.0f;
 //GetNodeAttributeOrDefault(t_tree, "noise_level", fNoiseLevel, fNoiseLevel);
 //if(fNoiseLevel < 0.0f) {
 //THROW_ARGOSEXCEPTION("Can't specify a negative value for the noise level of the foot-bot ground sensor");
 //}
 //else if(fNoiseLevel > 0.0f) {
 //m_bAddNoise = true;
 //m_cNoiseRange.Set(-fNoiseLevel, fNoiseLevel);
 //m_pcRNG = CRandom::CreateRNG("argos");
 //}
 m_tReadings.resize(4);
 }
 catch(CARGoSException& ex) {
 THROW_ARGOSEXCEPTION_NESTED("Initialization error in foot-bot phero sensor", ex);
 }
 }

 /****************************************/
 /****************************************/

 void CFootBotPheroSensor::Update() {
 /*
 * We make the assumption that the robot is rotated only wrt to Z
 */
 /* Get robot position and orientation */
 const CVector3& cEntityPos = m_pcEmbodiedEntity->GetOriginAnchor().Position;
 const CQuaternion& cEntityRot = m_pcEmbodiedEntity->GetOriginAnchor().Orientation;
 CRadians cRotZ, cRotY, cRotX;
 cEntityRot.ToEulerAngles(cRotZ, cRotY, cRotX);
 /* Set robot center */
 CVector2 cCenterPos(cEntityPos.GetX(), cEntityPos.GetY());
 /* Position of sensor on the ground after rototranslation */
 CVector2 cSensorPos;
 /* Go through the sensors */
 for(UInt32 i = 0; i < m_tReadings.size(); ++i) {
 /* Calculate sensor position on the ground */
 cSensorPos = m_pcPheroSensorEntity->GetSensor(i).Offset;
 cSensorPos.Rotate(cRotZ);
 cSensorPos += cCenterPos;
 /* Get the phero 20200602 - read from LoopFunction */
 const Real& cPhero = m_pcLoopFunctions.pheroMat(m_pcLoopFunctions.PosToIdx(cSensorPos.GetX(), 100, 2.5),
 m_pcLoopFunctions.PosToIdx(cSensorPos.GetX(), 100, 2.5));
 /* Set the reading */
 m_tReadings[i].Value = cPhero;
 /* Apply noise to the sensor - commented out 20200609 */
//  if(m_bAddNoise) {
//  m_tReadings[i].Value += m_pcRNG->Uniform(m_cNoiseRange);
//  }
 /* Set the final reading 20200602 commented */
  //m_tReadings[i].Value = m_tReadings[i].Value < 0.5f ? 0.0f : 1.0f;
 }
 }

 /****************************************/
 /****************************************/

 void CFootBotPheroSensor::Reset() {
 for(UInt32 i = 0; i < GetReadings().size(); ++i) {
 m_tReadings[i].Value = 0.0f;
 }
 }

 /****************************************/
 /****************************************/

 REGISTER_SENSOR(CFootBotPheroSensor,
 "footbot_phero", "default",
 "Seongin Na [seongin.na03@gmail.com]",
 "1.0",
 "The foot-bot pheromone detection sensor.",
 "This sensor accesses the foot-bot pheromone sensor. For a complete description\n"
 "of its usage\n",
 "Usable"
 );

}